<?php

  // L'astuce est la : utiliser php://input (PAS stdin qui ne fonctionne pas)
  // php://input recupere tout ce qui vient sur la liaison. On ne peut passer 
  // qu'un fichier a la fois, mais c'est moins gourmand en RAM (qui explose de 
  // toutes facons des que le fichier source est trop gros).
  // Par contre, toutce qui arrive dans input est le fichier. Il est donc 
  // impossible de passer plusieurs parametres en POST. Tous les parametres 
  // doivent arriver par GET !

  // Pour que le systeme ne loggue pas l'erreur si le fichier passe est plus
  // gros que post_max_size, il faut mettre un .htaccess avec 
  //   php_value display_errors off
  // Le log se fait sinon dans tous les cas a l'ecran et corrompt la 
  // communication XML
  require ("functions.php");
  require ("Net/smtp.php");
  require ("LDAP/chercher.php");
  require ("Strings/strings.php");

  ///////////////////////////
  //   PARAMETERS CHECKS   //
  ///////////////////////////
  if (isset ($_GET['FU']))
    $name = basename ($_GET['FU']);
  else
  {
    trigger_error ("FU not given !", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("FU not given !"));
    die (json_encode ($result));
  }

  if (isset ($_GET['sid']))
    $sid = $_GET['sid'];
  else
  {
    trigger_error ("sid not given !", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("sid not given !"));
    die (json_encode ($result));
  }

  if (checkShortID ($sid) === FALSE)
  {
    trigger_error ("Sid '$sid' contains illegal characters", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("SID contains illegal characters"));
    die (json_encode ($result));
  }

  if (checkFileName ($name) === FALSE)
  {
    trigger_error ("Filename '$name' contains illegal characters",
                   E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("Filename contains illegal characters"));
    die (json_encode ($result));
  }

  if (! file_exists ($conf['uploadTmp'] ) || ! is_dir ($conf['uploadTmp']) ||
      ! is_writeable ($conf['uploadTmp']))
  {
    // Last test : if the temp dir is not correct, it is fatal on the index 
    // page
    trigger_error ("Temp dir '".$conf['uploadTmp']."' is incorrect",
                   E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("Temp dir is incorrect"));
    die (json_encode ($result));
  }
  if (! file_exists ($conf['uploadDirUserBase'] ) ||
      ! is_dir ($conf['uploadDirUserBase']) ||
      ! is_writeable ($conf['uploadDirUserBase']))
  {
    // Last test : if the UserDir is not correct, it is fatal on the index 
    // page
    trigger_error ("User dir '".$conf['uploadDirUserBase']."' is incorrect",
                   E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("UserDir is incorrect"));
    die (json_encode ($result));
  }

  //////////////////////////////////////////////////////////////////////
  //   LOOK FOR THE INFORMATIONS BASED ON THE SID IN MySQL AND LDAP   //
  //////////////////////////////////////////////////////////////////////
  $sql = "SELECT longID,uid,mail FROM auth WHERE shortID='$sid' AND Allow=1";
  $result = mysql_query ($sql);
  if ($result === FALSE)
  {
    trigger_error ("SQL ERROR : $sql", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("SQL ERROR"));
    die (json_encode ($result));
  }
  if (mysql_num_rows ($result) !== 1)
  {
    trigger_error ("Not found '$sid' in DB or Allow=0", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("ShortID not found or don't accept files"));
    die (json_encode ($result));
  }
  $res = mysql_fetch_assoc($result);

  $uid = $res['uid'];
  $longID = $res['longID'];
  $mail = $res['mail'];

  if ($mail === "")
  {
    // Mail is not available in MySQL, use LDAP connection
    $ret = chercher ($conf['ldap_base'], "(uid=$uid)", array ("mail"),
                     $conf['ldap_server'], $conf['ldap_port'],
                     $conf['ldap_bindDN'], $conf['ldap_bindPassword']);

    if (is_string ($ret) || ! isset ($ret[0]['mail'][0]))
    {
      trigger_error ("LDAP Error : $ret", E_USER_ERROR);
      $result = array ('success'=>FALSE,
                       'error'=> t ("LDAP ERROR : mail not found"));
      trigger_error ("LDAP Error : User '$uid' don't have mail in LDAP",
                     E_USER_ERROR);
      die (json_encode ($result));
    }
      
    $mail = $ret[0]['mail'][0];
  }

  //////////////////////
  //   CHECK QUOTAS   //
  //////////////////////
  $definedQuotas = FileStatsDefinedQuotaUser ($uid);
  $usedQuotas = FileStatsUsedQuotaUser ($uid);

  ////////////////////////////
  //   RECORD OF THE FILE   //
  ////////////////////////////
  $fpIn = fopen ("php://input", "r");
  if ($fpIn === FALSE)
  {
    trigger_error ("php://input non openable for reading", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("php://input non openable"));
    die (json_encode ($result));
  }

  $temp = tempnam ($conf['uploadTmp'], "FU-");
  $fpOut = fopen ($temp, "w");
  if ($fpOut === FALSE)
  {
    trigger_error ("'$temp' non openable for writing", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("Temp file non openable for writing"));
    die (json_encode ($result));
  }

  trigger_error ("Start Upload '$name' for '$uid'");
  $size = 0;
  // Each $sizeQuota, updateQuota to be protected if 10 upload are done
  // simultaneously
  $sizeQuotaLimit = 10*1024*1024;
  $sizeQuota = 0;
  set_time_limit (91);
  while ($datas = fread ($fpIn, 1024))
  {
    fwrite ($fpOut, $datas);
    $size+= strlen ($datas);

    if ($sizeQuota > $sizeQuotaLimit)
    {
      // Update Quotas time to time
      FileStatsAddFile ($uid, $temp, $size);
      // Re-read Quota values
      $usedQuotas = FileStatsUsedQuotaUser ($uid);
      $sizeQuota = 0;
    }
    // Check Quotas
    if ($definedQuotas['quotaSize'] !== NULL && 
        $definedQuotas['quotaSize'] < $usedQuotas['quotaSize']+$size)
    {
      // QUOTA SIZE EXCEEDED
      fclose ($fpOut);
      fclose ($fpIn);
      unlink ($temp);
      trigger_error ("Quota Size exceeded for '$uid' : ".
                     "Size=".($usedQuotas['quotaSize']+$size)."/".
                             $definedQuotas['quotaSize'],
                     E_USER_ERROR);
      $result = array ('success'=>FALSE,
                       'error'=> t ("Size Quota exceeded for this user"));
      die (json_encode ($result)); 
    }

    if ( $definedQuotas['quotaNumber'] !== NULL &&
        $definedQuotas['quotaNumber'] < $usedQuotas['quotaNumber'])
    {
      // QUOTA NUMBER EXCEEDED
      fclose ($fpOut);
      fclose ($fpIn);
      unlink ($temp);
      trigger_error ("Quota Number exceeded for '$uid' : ".
                     "Number=".$usedQuotas['quotaNumber']."/".
                               $definedQuotas['quotaNumber'],
                     E_USER_ERROR);
      $result = array ('success'=>FALSE,
                       'error'=> t ("Quota number exceeded for this user"));
      die (json_encode ($result)); 
    }
    set_time_limit (91);
  }
  fclose ($fpOut);
  fclose ($fpIn);
  trigger_error ("End Upload '$name' for '$uid'");
  // Add Quotas at least one time
  FileStatsAddFile ($uid, $temp, $size);
  $fileName = $conf['uploadDirUserBase']."/$longID/$name";
  $pathinfo = pathinfo ($fileName);
  while (file_exists ($fileName))
  {
    $fileName  = $conf['uploadDirUserBase']."/$longID/".$pathinfo['filename'].
                 "-".rand (10,99);
    if (isset ($pathinfo['extension']) && $pathinfo['extension'] !== "")
      $fileName .= ".".$pathinfo['extension'];
  }
  $file = substr ($fileName, strlen ($conf['uploadDirUserBase']."/$longID/"));
  if (!file_exists ($conf['uploadDirUserBase']."/$longID/"))
  {
    trigger_error ("Create '".$conf['uploadDirUserBase']."/$longID/");
    mkdir ($conf['uploadDirUserBase']."/$longID/");
  }
  if ($conf['debug'] > 0)
    trigger_error ("Put '$temp' in '$fileName'", E_USER_NOTICE);
  rename ($temp, $fileName);
  FileStatsRenFile ($temp, "/$longID/$file");

  ///////////////////////////
  //   SEND MAIL TO USER   //
  ///////////////////////////
  $definedQuotas = FileStatsDefinedQuotaUser ($uid);
  $usedQuotas = FileStatsUsedQuotaUser ($uid);
  if ($definedQuotas['quotaSize'] === NULL)
    $definedQuotas['quotaSize'] = "NO LIMIT";
  else
    $definedQuotas['quotaSize'] = HumanReadableValue ($definedQuotas['quotaSize']);
  if ($definedQuotas['quotaNumber'] === NULL)
    $definedQuotas['quotaNumber'] = "NO LIMIT";
  $downloadURL = $conf['smtpDownloadURL']. "/$longID/".urlencode ($file);
  $downloadURLClear = $conf['smtpDownloadURL']."/$longID/$file";
  $fileMail = "themes/".$conf['wwwTheme']."/mail.php";
  $contents = file_get_contents ($fileMail);
  // Replace special strings
  $contents = str_replace ("<CONTACT>", $conf['wwwFooter']['us'], $contents);
  $contents = str_replace ("<TITLEU2>", $conf['wwwTitle']['us'], $contents);
  $contents = str_replace ("<STYLEU2>",
                   file_get_contents ("themes/".$conf['wwwTheme']."/style.css"),
                   $contents);
  $contents = str_replace ("<DOWNLOADURL>", $downloadURL, $contents);
  $contents = str_replace ("<DOWNLOADURLCLEAR>", $downloadURLClear, $contents);
  $contents = str_replace ("<FILELIFETIME>", $conf['filesLifetime'], $contents);
  $contents = str_replace ("<DEFINEDQUOTASIZE>", $definedQuotas['quotaSize'],
                           $contents);
  $contents = str_replace ("<DEFINEDQUOTANUMBER>",
                           $definedQuotas['quotaNumber'], $contents);
  $contents = str_replace ("<USEDQUOTASIZE>",
                           HumanReadableValue ($usedQuotas['quotaSize']),
                           $contents);
  $contents = str_replace ("<FILESIZE>", HumanReadableValue ($size), $contents);
  $contents = str_replace ("<USEDQUOTANUMBER>",
                           $usedQuotas['quotaNumber'], $contents);
  $contents = str_replace ("<QUICKURL>", $conf['smtpQuickURL'].$sid, $contents);
  $contents = str_replace ("<SITEURL>", $conf['smtpQuickURL'], $contents);
  trigger_error ("Send mail to '$mail' for '$fileName'", E_USER_NOTICE);
  $rc = smtp_mail ($mail, $conf['smtp_from'], $conf['smtp_subject'], $contents,
                   $conf['smtp_host'], $conf['smtp_attached'], "",
                   $conf['smtp_user'], $conf['smtp_pass']);

  if ($rc !== TRUE)
  {
    trigger_error ("Mail could not be send to '$mail'", E_USER_ERROR);
    $result = array ('success'=>FALSE,
                     'error'=> t ("Mail could not be send to final user"));
    die (json_encode ($result));
  }

  //////////////////////////
  //   END OF PROCESSUS   //
  //////////////////////////
  $result = array ('success'=>TRUE,
                   'error'=> t ("Upload is correctely terminated and an email".
                                " is sent to the user"));
  die (json_encode ($result));
?>
